//Code by Ashish Siddharth
//28-April-2019

if ( ! Detector.webgl ) Detector.addGetWebGLMessage();

var camera, scene, renderer;
var gui, parameters, grid, id, model;
const threeGlasses = new THREE.Object3D();
const loader = new THREE.GLTFLoader();
init();

function init() {

  scene = new THREE.Scene();

  //ambient light
  scene.add( new THREE.AmbientLight( 0xffffff, 2 ) );
  
  //hemispherical light
  var light = new THREE.HemisphereLight( 0xffffbb, 0x080820, 1 );
  scene.add( light );
  
  //point light
  var pointLight = new THREE.PointLight( 0xffffff, 2 );
  scene.add( pointLight );
  
  //multiple point lights
  var lights = [];
  lights[ 0 ] = new THREE.PointLight( 0xffffff, 1, 0 );
  lights[ 1 ] = new THREE.PointLight( 0xffffff, 1, 0 );
  lights[ 2 ] = new THREE.PointLight( 0xffffff, 1, 0 );

  lights[ 0 ].position.set( 0, 10, 0 );
  lights[ 1 ].position.set( 30, 20, 10 );
  lights[ 2 ].position.set( -30, - 20, - 10, 1 );

  scene.add( lights[ 0 ] );
  scene.add( lights[ 1 ] );
  scene.add( lights[ 2 ] );

  //direction light
  var directionalLight = new THREE.DirectionalLight( 0xffffff, 1 );
  scene.add( directionalLight );
  
  //grid
  grid = new THREE.GridHelper( 25, 50, 0xffffff, 0x555555 );
  grid.rotateOnAxis( new THREE.Vector3( 1, 0, 0 ), 90 * ( Math.PI/180 ) );
  scene.add(grid);
  
  //camera
  camera = new THREE.PerspectiveCamera( 75, window.innerWidth / window.innerHeight, 1, 1000 );
  camera.up.set( 0, 0, 1 );   // Z is up for objects intended to be 3D printed.
  camera.position.set( 0, -12, 10 );
  camera.add( new THREE.PointLight( 0xffffff, 0.8 ) );
  scene.add( camera );

  renderer = new THREE.WebGLRenderer( { antialias: true } );
  renderer.setClearColor( 0x999999 );
  renderer.setPixelRatio( window.devicePixelRatio );
  renderer.setSize( window.innerWidth, window.innerHeight );
  renderer.gammaOutput = false;
  document.body.appendChild( renderer.domElement );
  
  //load model
	addmodel();
	
  //load control
	addControl();
  
  //mouse controls
  controls = new THREE.OrbitControls( camera, renderer.domElement );
  controls.addEventListener( 'change', render );
  controls.target.set( 0, 1.2, 2 );
  controls.update();
  
  window.addEventListener( 'resize', onWindowResize, false );
  window.addEventListener( 'mousedown', onMouseClick, false );  
  window.addEventListener( 'mousemove', onMouseMove, false ); 
}

//load glasses
function addmodel() {
	loader.load( './model/superstar6.glb', function (modelGeometry){
		model = modelGeometry.scene;
		model.traverse ( ( o ) => {
			if ( o.isMesh ) {
			o.material.metalness = true;
			o.material.roughness = 0.7;
			o.material.wireframe = false;
			o.material.refractionRatio = 0.95;
			}
		} );
		threeGlasses.add(model);
		threeGlasses.scale.set( 0.08, 0.08, 0.08);
		threeGlasses.rotation.x = 70*Math.PI/180;
	});
	scene.add(threeGlasses);
}

//gui creation, called by the init
function addControl() {
	gui = new dat.GUI({ width: 300});
	gui.open();
	
	//display
	disp_param = 
	{
		Grid: false,
		GammaEncoding: false,
		WireFrame: false,
	};
	var dispFolder = gui.addFolder( 'Display');
	
	dispFolder.add( disp_param, 'Grid' ).onChange( function ( val ) {
		if (val == true){
			grid.rotation.x=70*Math.PI/180;
			scene.add(grid);
			render();
		} 
		else {
			scene.remove(grid);
			render();
		}
	} );
	
	dispFolder.add( disp_param, 'GammaEncoding' ).onChange( function ( val ) {
		if (val == true){
			renderer.gammaOutput = true;
			render();
		} 
		else {
			renderer.gammaOutput = false;
			render();
		}
	} );
	
	dispFolder.add( disp_param, 'WireFrame' ).onChange( function ( val ) {
		if (val == true){
			//blank
		} 
		else {
			//blank
		}
	} );
	
	//animate model
	ani_param = 
	{
		AnimateZ: false,
	};
	var aniFolder = gui.addFolder( 'Animate');
	
	aniFolder.add( ani_param, 'AnimateZ' ).onChange( function (val) {
		if (val == true){
			animate();
		}
		else{
			cancelAnimationFrame( id );
			threeGlasses.rotation.y=0;
			render();
		}
	} );
}

//animation function
function animate(val) {
	id = requestAnimationFrame( animate );
	threeGlasses.rotation.y += 0.01;
	renderer.render( scene, camera );
}

//to adjust scene on windows resize
function onWindowResize() {
  camera.aspect = window.innerWidth / window.innerHeight;
  camera.updateProjectionMatrix();
  renderer.setSize( window.innerWidth, window.innerHeight );
  render();		//render funcn is call here so that on resize, screen doesn't black out
}


function onMouseClick() {
  render();		//render on mouse click
}

function onMouseMove() {
  render();		//render on mouse click
}

function render() {
  renderer.render( scene, camera );
}